/*******************************************************************************
 * Copyright (c) 2004, 2007 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/
package org.eclipse.birt.data.engine.olap.data.impl.aggregation.filter;

import java.util.logging.Logger;

import org.eclipse.birt.data.engine.olap.api.query.ICubeFilterDefinition;
import org.eclipse.birt.data.engine.olap.api.query.ILevelDefinition;
import org.eclipse.birt.data.engine.olap.data.api.DimLevel;
import org.eclipse.birt.data.engine.olap.util.filter.IJSTopBottomFilterHelper;


/**
 *
 */
public class TopBottomFilter
{
	private IJSTopBottomFilterHelper filterHelper;
	private double n;
	private int filterType; 
	private DimLevel[] aggrLevels;
	private DimLevel targetLevel;
	private DimLevel[] axisQualifierLevels;
	private Object[] axisQualifierValues;
	private static Logger logger = Logger.getLogger( TopBottomFilter.class.getName( ) );
	
	/**
	 * @deprecated Just used in test cases, and use TopBottomFilterDefn(
	 *             TopBottomFilterEvalHelper filterHelper ) instead.
	 * @param n
	 */
	public TopBottomFilter( int n )
	{
		logger.entering( TopBottomFilter.class.getName( ),
				"TopBottomFilterDefn",
				new Integer( n ) );
		if ( n == 0 )
			throw new IllegalArgumentException( "Parameter n can't be zero!" );
		this.n = n > 0 ? n : -n;
		filterType = IJSTopBottomFilterHelper.TOP_N;
		logger.exiting( TopBottomFilter.class.getName( ),
				"TopBottomFilterDefn" );
	}

	/**
	 * @deprecated Just used in test cases, and use TopBottomFilterDefn(
	 *             TopBottomFilterEvalHelper filterHelper ) instead.
	 * @param n
	 * @param targerLevel
	 * @param aggrLevels
	 */
	public TopBottomFilter( int n, DimLevel targerLevel,
			DimLevel[] aggrLevels )
	{
		this( n );
		Object[] params = {
				new Integer( n ), targerLevel, aggrLevels
		};
		logger.entering( TopBottomFilter.class.getName( ),
				"TopBottomFilterDefn",
				params );
		this.targetLevel = targerLevel;
		this.aggrLevels = aggrLevels;
		logger.exiting( TopBottomFilter.class.getName( ),
				"TopBottomFilterDefn" );
	}

	/**
	 * 
	 * @param filterHelper
	 */
	public TopBottomFilter( IJSTopBottomFilterHelper filterHelper )
	{
		ICubeFilterDefinition cubeFilter = filterHelper.getCubeFilterDefinition( );
		this.aggrLevels = filterHelper.getAggrLevels( );
		filterType = filterHelper.getFilterType( );
		this.n = filterHelper.getN();
		this.filterHelper = filterHelper;
		this.targetLevel = new DimLevel(cubeFilter.getTargetLevel( ));
		ILevelDefinition[] axisLevels = cubeFilter.getAxisQualifierLevels( );
		if ( axisLevels != null )
		{
			axisQualifierLevels = new DimLevel[axisLevels.length];
			for ( int i = 0; i < axisLevels.length; i++ )
			{
				axisQualifierLevels[i] = new DimLevel(axisLevels[i]);
			}
		}
		axisQualifierValues = cubeFilter.getAxisQualifierValues( );
	}

	/**
	 * @return the n, which will be greater than zero.
	 */
	public double getN( )
	{
		return n;
	}
	
	
	/**
	 * @return the filterType
	 */
	public int getFilterType( )
	{
		return filterType;
	}

	/**
	 * @return the aggrLevels
	 */
	public DimLevel[] getAggrLevels( )
	{
		return aggrLevels;
	}
	
	/**
	 * @param aggrLevels the aggrLevels to set
	 */
	public void setAggrLevels( DimLevel[] aggrLevels )
	{
		this.aggrLevels = aggrLevels;
	}
	
	/**
	 * @return the targerLevelName
	 */
	public DimLevel getTargetLevel( )
	{
		return targetLevel;
	}
	
	/**
	 * @param targerLevel the targerLevelName to set
	 */
	public void setTargetLevel( DimLevel targerLevel )
	{
		this.targetLevel = targerLevel;
	}

	
	/**
	 * @return the filterHelper
	 */
	public IJSTopBottomFilterHelper getFilterHelper( )
	{
		return filterHelper;
	}

	
	/**
	 * @return the axisQualifierLevels
	 */
	public DimLevel[] getAxisQualifierLevels( )
	{
		return axisQualifierLevels;
	}

	
	/**
	 * @return the axisQualifierValues
	 */
	public Object[] getAxisQualifierValues( )
	{
		return axisQualifierValues;
	}
	
	
}
