/*******************************************************************************
 * Copyright (c) 2004, 2005 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/
package org.eclipse.birt.data.engine.olap.query.view;

import javax.olap.OLAPException;

import org.eclipse.birt.data.engine.core.DataException;
import org.eclipse.birt.data.engine.i18n.ResourceConstants;

/**
 * MeasureNameManager class is to search measure index based on its measure name
 * 
 */
public class MeasureNameManager
{
	private CalculatedMember[] members;

	/**
	 * 
	 * @param members
	 */
	MeasureNameManager( CalculatedMember[] members )
	{
		this.members = members;
	}

	/**
	 * 
	 * @param index
	 * @return
	 * @throws OLAPException
	 */
	public String getAggrName( int index ) throws DataException
	{
		if ( index >= this.members.length || index < 0 )
			throw new DataException( ResourceConstants.MEASURE_NAME_NOT_FOUND );
		return this.members[index].getName( );
	}
	
	/**
	 * 
	 * @param name
	 * @return
	 * @throws DataException
	 */
	public int getAggregationResultID( String name ) throws DataException
	{
		int rsID = -1;
		for ( int i = 0; i < this.members.length; i++ )
		{
			if ( this.members[i].getName( ).equals( name ) )
			{
				rsID = this.members[i].getRsID( );
				break;
			}
		}
		if ( rsID == -1 )
			throw new DataException( ResourceConstants.CANNOT_GET_MEASURE_VALUE,
					new Object[]{
						name
					} );
		else
			return rsID;
	}
	
	/**
	 * 
	 * @param name
	 * @return
	 * @throws DataException 
	 */
	public int getAggregationIndex( String name ) throws DataException
	{
		int rsID = this.getAggregationResultID( name );
		int index = 0;
		if ( rsID >= 0 )
		{

			for ( int i = 0; i < this.members.length; i++ )
			{
				if ( this.members[i].getName( ).equals( name ) )
					break;
				if ( this.members[i].getRsID( ) == rsID
						&& !this.members[i].getName( ).equals( name ) )
				{
					index++;
				}
			}
		}
		return index;
	}
	
	/**
	 * 
	 * @param name
	 * @return
	 * @throws DataException
	 */
	public CalculatedMember getCalculatedMember( String name )
			throws DataException
	{
		for ( int i = 0; i < this.members.length; i++ )
		{
			if ( this.members[i].getName( ).equals( name ) )
			{
				return this.members[i];
			}
		}
		throw new DataException( ResourceConstants.CANNOT_GET_MEASURE_VALUE,
				new Object[]{
					name
				} );
	}
	
	/**
	 * 
	 * @return
	 */
	public CalculatedMember[] getCalculatedMembers( )
	{
		return this.members;
	}
}
