/*******************************************************************************
 * Copyright (c) 2005 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.report.designer.data.ui.dataset;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.birt.core.exception.BirtException;
import org.eclipse.birt.data.engine.api.aggregation.IAggregationFactory;
import org.eclipse.birt.data.engine.api.aggregation.IAggregationInfo;
import org.eclipse.birt.data.engine.api.aggregation.IParameterInfo;
import org.eclipse.birt.report.designer.data.ui.property.AbstractDescriptionPropertyPage;
import org.eclipse.birt.report.designer.data.ui.util.ControlProvider;
import org.eclipse.birt.report.designer.data.ui.util.DataSetExpressionProvider;
import org.eclipse.birt.report.designer.data.ui.util.DataSetProvider;
import org.eclipse.birt.report.designer.data.ui.util.DataUtil;
import org.eclipse.birt.report.designer.data.ui.util.Utility;
import org.eclipse.birt.report.designer.internal.ui.util.ExceptionHandler;
import org.eclipse.birt.report.designer.nls.Messages;
import org.eclipse.birt.report.designer.ui.ReportPlugin;
import org.eclipse.birt.report.designer.ui.dialogs.ExpressionBuilder;
import org.eclipse.birt.report.designer.util.DEUtil;
import org.eclipse.birt.report.model.api.AggregationArgumentHandle;
import org.eclipse.birt.report.model.api.ColumnHintHandle;
import org.eclipse.birt.report.model.api.ComputedColumnHandle;
import org.eclipse.birt.report.model.api.DataSetHandle;
import org.eclipse.birt.report.model.api.DesignElementHandle;
import org.eclipse.birt.report.model.api.PropertyHandle;
import org.eclipse.birt.report.model.api.StructureFactory;
import org.eclipse.birt.report.model.api.activity.SemanticException;
import org.eclipse.birt.report.model.api.elements.DesignChoiceConstants;
import org.eclipse.birt.report.model.api.elements.structures.AggregationArgument;
import org.eclipse.birt.report.model.api.elements.structures.ComputedColumn;
import org.eclipse.birt.report.model.api.metadata.IChoice;
import org.eclipse.birt.report.model.core.Structure;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.IMessageProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;

/**
 * TODO: Please document
 * 
 * @version $Revision: 1.24.2.1 $ $Date: 2007/09/04 06:45:57 $
 */
public class DataSetComputedColumnsPage extends AbstractDescriptionPropertyPage
		implements
			ITableLabelProvider
{

	private transient PropertyHandle computedColumns = null;
	private transient PropertyHandleTableViewer viewer = null;
	private static String[] cellLabels = new String[]{
			Messages.getString( "dataset.editor.title.columnName" ),//$NON-NLS-1$
			Messages.getString( "dataset.editor.title.dataType" ),//$NON-NLS-1$
			Messages.getString( "dataset.editor.title.expression" ),//$NON-NLS-1$
			Messages.getString( "dataset.editor.title.aggrFunc" ),
			Messages.getString( "dataset.editor.title.aggrArgu" ),
			Messages.getString( "dataset.editor.title.filter" )
	};

	private static String[] cellProperties = new String[]{
			ComputedColumn.NAME_MEMBER,
			ComputedColumn.DATA_TYPE_MEMBER,
			ComputedColumn.EXPRESSION_MEMBER,
			ComputedColumn.AGGREGATEON_FUNCTION_MEMBER,
			ComputedColumn.ARGUMENTS_MEMBER,
			ComputedColumn.FILTER_MEMBER
	};

	private static IChoice[] dataTypes = DEUtil.getMetaDataDictionary( )
			.getStructure( ComputedColumn.COMPUTED_COLUMN_STRUCT )
			.getMember( ComputedColumn.DATA_TYPE_MEMBER )
			.getAllowedChoices( )
			.getChoices( );

	private static List aggrFuncs ;

	public static String[] funcNames;

	public static Map funcArgMap = new HashMap( );

	private static int maxArgNum = 0;
	static
	{
		try
		{
			aggrFuncs = DataUtil.getAggregationFactory()
					.getAggrInfoList( IAggregationFactory.AGGR_TABULAR );
			funcNames = new String[aggrFuncs.size( )];
			int index = 0;
			for ( Iterator i = aggrFuncs.iterator( ); i.hasNext( ); index++ )
			{
				IAggregationInfo function = (IAggregationInfo) i.next( );
				funcNames[index] = function.getName( );
				List parameters = function.getParameters( );
				funcArgMap.put( function.getName( ), parameters );
				if ( parameters.size( ) > maxArgNum )
				{
					maxArgNum = parameters.size( );
				}
			}
		}
		catch ( BirtException e )
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * 
	 */
	public DataSetComputedColumnsPage( )
	{
		super( );
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.report.designer.ui.dialogs.properties.IPropertyPage#createPageControl(org.eclipse.swt.widgets.Composite)
	 */
	public Control createContents( Composite parent )
	{
		computedColumns = ( (DataSetHandle) getContainer( ).getModel( ) ).getPropertyHandle( DataSetHandle.COMPUTED_COLUMNS_PROP );;
		viewer = new PropertyHandleTableViewer( parent, true, true, true );
		TableColumn column = new TableColumn( viewer.getViewer( ).getTable( ),
				SWT.LEFT );
		column.setText( cellLabels[0] );
		column.setWidth( 150 );
		column = new TableColumn( viewer.getViewer( ).getTable( ), SWT.LEFT );
		column.setText( cellLabels[1] );
		column.setWidth( 200 );
		column = new TableColumn( viewer.getViewer( ).getTable( ), SWT.LEFT );
		column.setText( cellLabels[2] );
		column.setWidth( 200 );
		column = new TableColumn( viewer.getViewer( ).getTable( ), SWT.LEFT );
		column.setText( cellLabels[3] );
		column.setWidth( 200 );
		viewer.getViewer( )
				.setContentProvider( new IStructuredContentProvider( ) {

					public Object[] getElements( Object inputElement )
					{
						ArrayList computedColumnsList = new ArrayList( 10 );
						Iterator iter = computedColumns.iterator( );
						if ( iter != null )
						{
							while ( iter.hasNext( ) )
							{
								computedColumnsList.add( iter.next( ) );
							}
						}
						return computedColumnsList.toArray( );
					}

					public void dispose( )
					{

					}

					public void inputChanged( Viewer viewer, Object oldInput,
							Object newInput )
					{

					}
				} );
		viewer.getViewer( ).setLabelProvider( this );
		viewer.getViewer( ).setInput( computedColumns );
		addListeners( );
		setToolTips( );
		return viewer.getControl( );
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.report.designer.ui.dialogs.properties.IPropertyPage#pageActivated()
	 */
	public void pageActivated( )
	{
		refreshColumnNames( );
		getContainer( ).setMessage( Messages.getString( "dataset.editor.computedColumns" ),
				IMessageProvider.NONE ); // $-NON-NLS-1
		// //$NON-NLS-1$
		setPageProperties( );

		computedColumns = ( (DataSetHandle) getContainer( ).getModel( ) ).getPropertyHandle( DataSetHandle.COMPUTED_COLUMNS_PROP );;
		viewer.getViewer( ).setInput( computedColumns );
		viewer.getViewer( ).getTable( ).select( 0 );
	}

	/**
	 * Refresh columns meta data
	 */
	private void refreshColumnNames( )
	{
		( (DataSetEditor) this.getContainer( ) ).getCurrentItemModel( true,
				true );
	}

	private void addListeners( )
	{
		viewer.getNewButton( ).addSelectionListener( new SelectionAdapter( ) {

			public void widgetSelected( SelectionEvent e )
			{
				doNew( );
			}
		} );

		viewer.getEditButton( ).addSelectionListener( new SelectionAdapter( ) {

			public void widgetSelected( SelectionEvent e )
			{
				doEdit( );
			}
		} );

		viewer.getViewer( ).getTable( ).addMouseListener( new MouseAdapter( ) {

			public void mouseDoubleClick( MouseEvent e )
			{
				doEdit( );
			}
		} );

		viewer.getViewer( ).getTable( ).addKeyListener( new KeyListener( ) {

			public void keyPressed( KeyEvent e )
			{
			}

			public void keyReleased( KeyEvent e )
			{
				if ( e.keyCode == SWT.DEL )
				{
					setPageProperties( );
				}
			}

		} );

		viewer.getRemoveButton( )
				.addSelectionListener( new SelectionListener( ) {

					public void widgetSelected( SelectionEvent e )
					{
						setPageProperties( );
					}

					public void widgetDefaultSelected( SelectionEvent e )
					{
					}

				} );

		viewer.getRemoveMenuItem( )
				.addSelectionListener( new SelectionListener( ) {

					public void widgetSelected( SelectionEvent e )
					{
						setPageProperties( );
					}

					public void widgetDefaultSelected( SelectionEvent e )
					{
					}

				} );

		viewer.getRemoveAllMenuItem( )
				.addSelectionListener( new SelectionListener( ) {

					public void widgetSelected( SelectionEvent e )
					{
						setPageProperties( );
					}

					public void widgetDefaultSelected( SelectionEvent e )
					{
						widgetSelected( e );
					}
				} );

		viewer.getViewer( )
				.addSelectionChangedListener( new ViewerSelectionListener( ) );
	}

	private void doNew( )
	{
		doEdit( new ComputedColumn( ) );
	}

	private void doEdit( )
	{
		int index = viewer.getViewer( ).getTable( ).getSelectionIndex( );
		if ( index == -1 )
			return;

		ComputedColumnHandle handle = (ComputedColumnHandle) viewer.getViewer( )
				.getTable( )
				.getItem( index )
				.getData( );

		doEdit( handle );
	}

	private void doEdit( Object structureOrHandle )
	{
		ComputedColumnInputDialog dlg = new ComputedColumnInputDialog( structureOrHandle );

		if ( dlg.open( ) == Window.OK )
		{
			update( structureOrHandle );
		}
	}
	
	private void update( Object structureOrHandle )
	{
		if ( structureOrHandle instanceof ComputedColumn )
		{
			try
			{
				computedColumns.addItem( (ComputedColumn) structureOrHandle );
				viewer.getViewer( ).refresh( );
			}
			catch ( SemanticException e )
			{
				ExceptionHandler.handle( e );
			}
		}
		else
		{
			viewer.getViewer( ).update( structureOrHandle, null );
		}

		if ( validateAllComputedColumns( ) )
			getContainer( ).setMessage( Messages.getString( "dataset.editor.computedColumns" ),
					IMessageProvider.NONE );
	}
	
	private ComputedColumn getStructure( Object structureOrHandle )
	{
		ComputedColumn structure = null;
		if ( structureOrHandle instanceof ComputedColumn )
		{
			structure = (ComputedColumn) structureOrHandle;
		}
		else
		{
			structure = (ComputedColumn) ( (ComputedColumnHandle) structureOrHandle ).getStructure( );
		}

		return structure;
	}
	
	private final String getTypeName( String typeDisplayName )
	{
		for ( int i = 0; i < dataTypes.length; i++ )
		{
			if ( dataTypes[i].getDisplayName( ).equals( typeDisplayName ) )
				return dataTypes[i].getName( );
		}

		return dataTypes[0].getName( );
	}

	private final String getTypeDisplayName( String typeName )
	{
		for ( int i = 0; i < dataTypes.length; i++ )
		{
			if ( dataTypes[i].getName( ).equals( typeName ) )
			{
				return dataTypes[i].getDisplayName( );
			}
		}

		return dataTypes[0].getDisplayName( );
	}
    
    private void setToolTips( )
	{
		viewer.getNewButton( )
				.setToolTipText( Messages.getString( "DataSetComputedColumnsPage.toolTipText.New" ) );//$NON-NLS-1$
		viewer.getEditButton( )
				.setToolTipText( Messages.getString( "DataSetComputedColumnsPage.toolTipText.Edit" ) );//$NON-NLS-1$
		viewer.getRemoveButton( )
				.setToolTipText( Messages.getString( "DataSetComputedColumnsPage.toolTipText.Remove" ) );//$NON-NLS-1$
		viewer.getUpButton( )
				.setToolTipText( Messages.getString( "DataSetComputedColumnsPage.toolTipText.Up" ) );//$NON-NLS-1$
		viewer.getDownButton( )
				.setToolTipText( Messages.getString( "DataSetComputedColumnsPage.toolTipText.Down" ) );//$NON-NLS-1$
	}
    
    /*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object,
	 *      int)
	 */
	public Image getColumnImage( Object element, int columnIndex )
	{
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object,
	 *      int)
	 */
	public String getColumnText( Object element, int columnIndex )
	{
		String value = null;
		ComputedColumn computedColumn = getStructure( element );

		try
		{
			switch ( columnIndex )
			{
				case 0 :
				{
					value = computedColumn.getName( );
					break;
				}
				case 1 :
				{
					value = getTypeDisplayName( computedColumn.getDataType( ) );
					break;
				}
				case 2 :
				{
					value = computedColumn.getExpression( );
					break;
				}
				case 3 :
				{
					value = computedColumn.getAggregateFunction( );
					for( int i = 0; i < aggrFuncs.size( ); i++ )
					{
						if( ((IAggregationInfo)aggrFuncs.get( i )).getName( ).equals( value ))
						{
							value = ((IAggregationInfo)aggrFuncs.get( i )).getDisplayName( );
							break;
						}
					}
					
					break;
				}
			}
		}
		catch ( Exception e )
		{
			ExceptionHandler.handle( e );
		}
		
		return ( value == null ? "" : value ); //$NON-NLS-1$
	}

    /*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#addListener(org.eclipse.jface.viewers.ILabelProviderListener)
	 */
	public void addListener( ILabelProviderListener listener )
	{
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#dispose()
	 */
	public void dispose( )
	{
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#isLabelProperty(java.lang.Object,
	 *      java.lang.String)
	 */
	public boolean isLabelProperty( Object element, String property )
	{
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#removeListener(org.eclipse.jface.viewers.ILabelProviderListener)
	 */
	public void removeListener( ILabelProviderListener listener )
	{
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.report.designer.ui.dialogs.properties.AbstractDescriptionPropertyPage#getPageDescription()
	 */
	public String getPageDescription( )
	{
		return Messages.getString( "DataSetComputedColumnsPage.description" ); //$NON-NLS-1$
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.report.designer.ui.dialogs.properties.AbstractPropertyPage#performOk()
	 */
	public boolean performOk( )
	{
		if ( validateAllComputedColumns( ) )
		{
			return super.performOk( );
		}
		return false;
	}

	private class ViewerSelectionListener implements ISelectionChangedListener
	{

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ISelectionChangedListener#selectionChanged(org.eclipse.jface.viewers.SelectionChangedEvent)
		 */
		public void selectionChanged( SelectionChangedEvent event )
		{
			// TODO Auto-generated method stub
			setPageProperties( );
		}

	}

	/**
	 * Depending on the Computed the properties of various controls on this page
	 * are set
	 */
	private void setPageProperties( )
	{
		boolean computedColumnsExist = false;

		computedColumnsExist = ( computedColumns != null
				&& computedColumns.getListValue( ) != null && computedColumns.getListValue( )
				.size( ) > 0 );
		viewer.getEditButton( ).setEnabled( computedColumnsExist );
		viewer.getDownButton( ).setEnabled( computedColumnsExist
				&& computedColumns.getListValue( ).size( ) > 1 );
		viewer.getUpButton( ).setEnabled( computedColumnsExist
				&& computedColumns.getListValue( ).size( ) > 1 );
		viewer.getRemoveButton( ).setEnabled( computedColumnsExist );
		viewer.getRemoveMenuItem( ).setEnabled( computedColumnsExist );
		viewer.getRemoveAllMenuItem( ).setEnabled( computedColumnsExist );
		validateAllComputedColumns( );
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.report.designer.ui.dialogs.properties.IPropertyPage#getToolTip()
	 */
	public String getToolTip( )
	{
		return Messages.getString( "DataSetComputedColumnsPage.ComputedColumns.Tooltip" ); //$NON-NLS-1$
	}

	public boolean canLeave( )
	{
		return validateAllComputedColumns( );
	}

	/*
	 * Exam whether all registered computed columns are valid. Set the message
	 * of dialog as appropriated
	 */
	private boolean validateAllComputedColumns( )
	{
		if ( computedColumns != null )
		{
			Iterator iter = computedColumns.iterator( );
			if ( iter != null )
			{
				while ( iter.hasNext( ) )
				{
					Object o = iter.next( );
					ComputedColumn computedColumn = null;
					if ( o instanceof ComputedColumnHandle )
					{
						computedColumn = (ComputedColumn) ( (ComputedColumnHandle) o ).getStructure( );
					}
					else
					{
						computedColumn = (ComputedColumn) o;
					}
					if ( !validateSingleColumn( computedColumn ) )
						return false;
				}
			}
		}
		return true;
	}

	/*
	 * Exam whether one computed column is valid or not.
	 */
	private boolean validateSingleColumn( ComputedColumn computedColumn )
	{
		if ( computedColumn.getName( ) == null
				|| computedColumn.getName( ).trim( ).length( ) == 0 )
		{
			getContainer( ).setMessage( Messages.getFormattedString( "dataset.editor.error.missingComputedColumnName", new Object[]{computedColumn.getName( )} ), IMessageProvider.ERROR ); //$NON-NLS-1$
			return false;
		}
		if ( computedColumn.getExpression( ) == null
				|| computedColumn.getExpression( ).trim( ).length( ) == 0 )
		{
			if ( isFunctionCount( computedColumn.getAggregateFunction( ) ) == false )
			{
				getContainer( ).setMessage( Messages.getFormattedString( "dataset.editor.error.missingComputedColumnExpression", new Object[]{computedColumn.getName( )} ), IMessageProvider.ERROR ); //$NON-NLS-1$
				return false;
			}
		}

		Iterator iter = ( (DataSetHandle) getContainer( ).getModel( ) ).getPropertyHandle( DataSetHandle.COLUMN_HINTS_PROP )
				.iterator( );
		while ( iter.hasNext( ) )
		{
			ColumnHintHandle hint = (ColumnHintHandle) iter.next( );
			if ( computedColumn.getName( ).equals( hint.getAlias( ) ) )
			{
				getContainer( ).setMessage( Messages.getFormattedString( "dataset.editor.error.computedColumnNameAlreadyUsed", new Object[]{computedColumn.getName( )} ), IMessageProvider.ERROR ); //$NON-NLS-1$
				return false;
			}
		}
		return true;
	}

	/**
	 * @param funcName
	 */
	private boolean isFunctionCount( String funcName )
	{
		return DesignChoiceConstants.AGGREGATION_FUNCTION_COUNT.equals( funcName )
				|| DesignChoiceConstants.AGGREGATION_FUNCTION_RUNNINGCOUNT.equals( funcName )
				|| DesignChoiceConstants.AGGREGATION_FUNCTION_COUNTDISTINCT.equals( funcName );
	}

	private class ComputedColumnInputDialog extends PropertyHandleInputDialog
	{

		private Text columnName = null;
		private Combo dataType = null;
		private Combo aggrCombo = null;
		private Text expression = null;
		private Label[] argLabels = null;
		private Text[] argTexts = null;
		private Button[] argButtons = null;
		private Label filterLabel = null;
		private Text filter = null;
		private Button filterButton = null;

		private ComputedColumnInputDialog( Object structureOrHandle )
		{
			super( structureOrHandle );
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.birt.report.designer.data.ui.dataset.PropertyHandleInputDialog#createCustomControls(org.eclipse.swt.widgets.Composite)
		 */
		protected void createCustomControls( Composite parent )
		{
			createTextCell( parent, 0 );
			createComboBoxCell( parent, 1 );
			createExpressionCell( parent, 2 );
			createAggrListCell( parent, 3 );
			createArgumentCell( parent, 4 );
			createFilterCell( parent, 5 );
		}

		private void createTextCell( Composite parent, final int index )
		{
			ControlProvider.createLabel( parent, cellLabels[index] );

			columnName = ControlProvider.createText( parent,
					(String) getProperty( getStructureOrHandle( ),
							cellProperties[index] ) );
			columnName.setLayoutData( ControlProvider.getGridDataWithHSpan( 2 ) );
			columnName.addModifyListener( new ModifyListener( ) {

				public void modifyText( ModifyEvent e )
				{
					validateSyntax( );
				}

			} );
		}

		private void createComboBoxCell( Composite parent, final int index )
		{
			ControlProvider.createLabel( parent, cellLabels[index] );

			dataType = ControlProvider.createCombo( parent, SWT.READ_ONLY );
			dataType.setLayoutData( ControlProvider.getGridDataWithHSpan( 2 ) );
			dataType.setItems( getDisplayNames( dataTypes ) );
			dataType.select( Utility.findIndex( dataType.getItems( ),
					getTypeDisplayName( (String) getProperty( getStructureOrHandle( ),
							cellProperties[index] ) ) ) );
			dataType.addSelectionListener( new SelectionAdapter( ) {

				public void widgetSelected( SelectionEvent e )
				{
					validateSyntax( );
				}

			} );
		}

		private void createAggrListCell( Composite parent, final int index )
		{
			ControlProvider.createLabel( parent, cellLabels[index] );

			aggrCombo = ControlProvider.createCombo( parent, SWT.READ_ONLY );
			aggrCombo.setLayoutData( ControlProvider.getGridDataWithHSpan( 2 ) );
			String[] aggrFunctionNames = getFuncDisplayNames( );
			aggrCombo.add( "" );
			for ( int i = 0; i < aggrFunctionNames.length; i++ )
				aggrCombo.add( aggrFunctionNames[i] );

			String aggrFuncName = (String) getProperty( getStructureOrHandle( ),
					cellProperties[index] );
			int index1 = 0;
			for ( int i = 1; i <= aggrFuncs.size( ); i++ )
			{
				if ( ( (IAggregationInfo) aggrFuncs.get( i - 1 ) ).getName( )
						.equals( aggrFuncName ) )
				{
					index1 = i;
				}
			}

			aggrCombo.select( index1 );
			aggrCombo.addSelectionListener( new SelectionAdapter( ) {

				public void widgetSelected( SelectionEvent e )
				{
					validateSyntax( );
					updateFilterUIStatus( );
					updateArgumentUIStatus( );
				}

			} );
		}
		
		/**
		 * @return whether filter component should be enabled
		 */
		private boolean needFilter( )
		{
			if ( aggrCombo.getText( ) != null
					&& aggrCombo.getText( ).trim( ).length( ) > 0 )
			{
				return true;
			}
			else
			{
				return false;
			}
		}
		
		/**
		 * Update the Filter component status
		 */
		private void updateFilterUIStatus( )
		{
			if ( needFilter( ) )
			{
				filter.setEnabled( true );
				filterLabel.setEnabled( true );
				filterButton.setEnabled( true );
			}
			else
			{
				filter.setText( "" );
				filter.setEnabled( false );
				filterLabel.setEnabled( false );
				filterButton.setEnabled( false );
			}
		}

		/**
		 * 
		 */
		private void updateArgumentUIStatus( )
		{
			List argInfoList = getArgInfoList( );
			int disableStart = 0;
			if ( argInfoList != null )
			{
				for ( int i = 0; i < argInfoList.size( ); i++ )
				{
					IParameterInfo argInfo = (IParameterInfo) argInfoList.get( i );
					argLabels[i].setText( argInfo.getDisplayName( ) );
					argTexts[i].setEnabled( true );
					argLabels[i].setEnabled( true );
					argButtons[i].setEnabled( true );
				}
				disableStart = argInfoList.size( );
			}
			for ( int i = disableStart; i < maxArgNum; i++ )
			{
				argLabels[i].setText( cellLabels[4] );
				argTexts[i].setText( "" );
				argTexts[i].setEnabled( false );
				argLabels[i].setEnabled( false );
				argButtons[i].setEnabled( false );
			}
		}
		
		/**
		 * @return
		 */
		private List getArgInfoList( )
		{
			String selectedFuncName = getSelectedFuncName( aggrCombo );
			if ( selectedFuncName == null )
				return null;
			return getFuncArgInfoList( selectedFuncName );
		}

		private void createArgumentCell( Composite parent, final int index )
		{
			argLabels = new Label[maxArgNum];
			argTexts = new Text[maxArgNum];
			argButtons = new Button[maxArgNum];
			for ( int i = 0; i < maxArgNum; i++ )
			{
				argLabels[i] = ControlProvider.createLabel( parent,
						cellLabels[index] );
				Composite composite = ControlProvider.getDefaultComposite( parent );
				Object handle = this.getStructureOrHandle( );
				if ( handle instanceof ComputedColumnHandle )
				{
					ComputedColumnHandle cHandle = (ComputedColumnHandle) handle;
					Iterator itr = cHandle.argumentsIterator( );
					int argIndex = 0;
					AggregationArgumentHandle argHandle = null;
					while ( itr.hasNext( ) && argIndex <= i )
					{// looking for the i-th argument handle
						argIndex++;
						argHandle = (AggregationArgumentHandle) itr.next( );
					}
					if ( argIndex <= i )
					{
						argHandle = null;
					}
					if ( argHandle == null )
						argTexts[i] = ControlProvider.createText( composite,
								null );
					else
						argTexts[i] = ControlProvider.createText( composite,
								argHandle.getValue( ) );

				}
				else
					argTexts[i] = ControlProvider.createText( composite,
							(String) getProperty( getStructureOrHandle( ),
									cellProperties[index] ) );
				argTexts[i].setLayoutData( ControlProvider.getGridDataWithHSpan( 1 ) );
				argTexts[i].addModifyListener( new ModifyListener( ) {

					public void modifyText( ModifyEvent e )
					{
						validateSyntax( );
					}

				} );

				final Text argText = argTexts[i];
				SelectionAdapter listener = new SelectionAdapter( ) {

					public void widgetSelected( SelectionEvent event )
					{
						ExpressionBuilder expressionBuilder = new ExpressionBuilder( argText.getText( ) );
						expressionBuilder.setExpressionProvier( new DataSetExpressionProvider( (DesignElementHandle) getContainer( ).getModel( ) ) );

						if ( expressionBuilder.open( ) == OK )
						{
							argText.setText( expressionBuilder.getResult( )
									.trim( ) );
						}
					}
				};

				argButtons[i] = ControlProvider.createButton( composite,
						SWT.PUSH,
						listener );
			}
			updateArgumentUIStatus( );
		}

		private void createFilterCell( Composite parent, final int index )
		{
			filterLabel = ControlProvider.createLabel( parent, cellLabels[index] );
 
			Composite composite = ControlProvider.getDefaultComposite( parent );
			Object handle = this.getStructureOrHandle( );
			if ( handle instanceof ComputedColumnHandle )
			{
				ComputedColumnHandle cHandle = (ComputedColumnHandle) handle;
				filter = ControlProvider.createText( composite,
						cHandle.getFilterExpression( ) );

			}
			else
				filter = ControlProvider.createText( composite,
						(String) getProperty( getStructureOrHandle( ),
								cellProperties[index] ) );
			filter.setLayoutData( ControlProvider.getGridDataWithHSpan( 1 ) );
			filter.setEnabled( false );
			filter.addModifyListener( new ModifyListener( ) {

				public void modifyText( ModifyEvent e )
				{
					validateSyntax( );
				}

			} );

			SelectionAdapter listener = new SelectionAdapter( ) {

				public void widgetSelected( SelectionEvent event )
				{
					ExpressionBuilder expressionBuilder = new ExpressionBuilder( filter.getText( ) );
					expressionBuilder.setExpressionProvier( new DataSetExpressionProvider( (DesignElementHandle) getContainer( ).getModel( ) ) );

					if ( expressionBuilder.open( ) == OK )
					{
						filter.setText( expressionBuilder.getResult( ).trim( ) );
					}
				}
			};

			filterButton = ControlProvider.createButton( composite, SWT.PUSH, listener );
			updateFilterUIStatus( );
		}

		private String[] getDisplayNames( IChoice[] choice )
		{
			String[] dataTypeDisplayNames = new String[choice.length];
			for ( int i = 0; i < choice.length; i++ )
			{
				dataTypeDisplayNames[i] = choice[i].getDisplayName( );
			}
			Arrays.sort( dataTypeDisplayNames );
			return dataTypeDisplayNames;

		}

		private String[] getFuncDisplayNames( )
		{
			String[] result = new String[aggrFuncs.size( )];
			for ( int i = 0; i < aggrFuncs.size( ); i++ )
			{
				result[i] = ( (IAggregationInfo) aggrFuncs.get( i ) ).getDisplayName( );
			}
			return result;
		}

		/**
		 * The aggrCombo contains the display names of all functions and "".
		 * 
		 * @param aggrCombo
		 * @return
		 */
		private String getSelectedFuncName( Combo aggrCombo )
		{
			int index = aggrCombo.getSelectionIndex( );
			if ( index == 0 )
				return null;
			return funcNames[index - 1];
		}

		private void createExpressionCell( Composite parent, final int index )
		{
			ControlProvider.createLabel( parent, cellLabels[index] );

			Composite composite = ControlProvider.getDefaultComposite( parent );
			expression = ControlProvider.createText( composite,
					(String) getProperty( getStructureOrHandle( ),
							cellProperties[index] ) );
			expression.setLayoutData( ControlProvider.getGridDataWithHSpan( 1 ) );
			expression.addModifyListener( new ModifyListener( ) {

				public void modifyText( ModifyEvent e )
				{
					validateSyntax( );
				}

			} );

			SelectionAdapter listener = new SelectionAdapter( ) {

				public void widgetSelected( SelectionEvent event )
				{
					ExpressionBuilder expressionBuilder = new ExpressionBuilder( expression.getText( ) );
					expressionBuilder.setExpressionProvier( new DataSetExpressionProvider( (DesignElementHandle) getContainer( ).getModel( ) ) );

					if ( expressionBuilder.open( ) == OK )
					{
						expression.setText( expressionBuilder.getResult( ).trim( ) );
					}
				}
			};

			ControlProvider.createButton( composite, SWT.PUSH, listener );
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.birt.report.designer.data.ui.dataset.PropertyHandleInputDialog#validateSemantics(java.lang.Object)
		 */
		protected IStatus validateSemantics( Object structureOrHandle )
		{
			AggregationArgument aggrArgument = null;
			String cName = columnName.getText( );
			String dataTypeName = getTypeName( dataType.getText( ) );
			String expr = expression.getText( );
			String funcName = getSelectedFuncName( this.aggrCombo );
			String filter = this.filter.getText( );
			
			try
			{
				if ( structureOrHandle instanceof ComputedColumnHandle )
				{
					ComputedColumnHandle handle = (ComputedColumnHandle) structureOrHandle;
					handle.setName( cName );
					handle.setDataType( dataTypeName );
					handle.setExpression( expr );
					handle.setAggregateFunction( funcName );
					handle.setFilterExpression( filter );
					handle.clearArgumentList( );
					List list = getFuncArgInfoList( funcName );
					if ( list != null )
					{
						for ( int i = 0; i < list.size( ); i++ )
						{
							aggrArgument = StructureFactory.createAggregationArgument( );
							IParameterInfo argInfo = (IParameterInfo) list.get( i );
							aggrArgument.setName( argInfo.getName( ) );
							aggrArgument.setValue( argTexts[i].getText( ).trim( ) );
							handle.addArgument( aggrArgument );
						}
					}
				}
				else if ( structureOrHandle instanceof ComputedColumn )
				{
					ComputedColumn handle = (ComputedColumn) structureOrHandle;
					handle.setName( cName );
					handle.setDataType( dataTypeName );
					handle.setExpression( expr );
					handle.setAggregateFunction( funcName );
					handle.setFilterExpression( filter );
					List list = getFuncArgInfoList( funcName );
					if ( list != null )
					{
						for ( int i = 0; i < list.size( ); i++ )
						{
							aggrArgument = StructureFactory.createAggregationArgument( );
							IParameterInfo argInfo = (IParameterInfo) list.get( i );
							aggrArgument.setName( argInfo.getName( ) );
							aggrArgument.setValue( argTexts[i].getText( ).trim( ) );
							handle.addArgument( aggrArgument );
						}
					}
				}
			}
			catch ( Exception e )
			{
				Status status = new Status( IStatus.ERROR,
						ReportPlugin.REPORT_UI,
						"",
						e );
				return status;
			}
			return getOKStatus( );
		}

		/**
		 * 
		 * @param funcName
		 * @return
		 */
		private List getFuncArgInfoList( String funcName )
		{
			return (List) funcArgMap.get( funcName );
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.birt.report.designer.data.ui.dataset.PropertyHandleInputDialog#validateSyntax(java.lang.Object)
		 */
		protected IStatus validateSyntax( Object structureOrHandle )
		{
			// duplicated columnName check
			if ( !isUniqueColumnName( ) )
				return getMiscStatus( IStatus.ERROR,
						Messages.getString( "DataSetComputedColumnsPage.duplicatedName" ) );

			// blankProperty check
			if ( isBlankProperty( columnName.getText( ) ) )
				return getBlankPropertyStatus( cellLabels[0] );
			if ( isBlankProperty( dataType.getText( ) ) )
				return getBlankPropertyStatus( cellLabels[1] );
			if ( isBlankProperty( expression.getText( ) ) )
			{
				String funcName = this.getSelectedFuncName( aggrCombo );
				if ( isFunctionCount( funcName ) == false )
					return getBlankPropertyStatus( cellLabels[2] );
			}
			//TODO: valida arguments
			/*if ( needArguments( ) && isBlankProperty( argument.getText( ) ) )
			{
				return getBlankPropertyStatus( cellLabels[4] );
			}*/
			return getOKStatus( );
		}

		private final boolean isUniqueColumnName( )
		{
			DataSetViewData[] items = DataSetProvider.getCurrentInstance( )
					.getColumns( ( (DataSetEditor) getContainer( ) ).getHandle( ),
							false,
							true,
							true );

			for ( int i = 0; i < items.length; i++ )
			{
				if ( !items[i].isComputedColumn( ) )
				{
					if ( ( items[i].getAlias( ) != null && items[i].getAlias( )
							.equals( columnName.getText( ) ) )
							|| ( items[i].getName( ) != null && items[i].getName( )
									.equals( columnName.getText( ) ) ) )
					{
						return false;
					}
				}
			}
			Iterator iter = computedColumns.iterator( );
			if ( iter != null )
			{
				while ( iter.hasNext( ) )
				{
					ComputedColumnHandle handle = (ComputedColumnHandle) iter.next( );
					if ( getStructure( getStructureOrHandle( ) ) != handle.getStructure( )
							&& handle.getName( ).equals( columnName.getText( ) ) )
					{
						return false;
					}
				}
			}
			Iterator iter2 = ( (DataSetHandle) getContainer( ).getModel( ) ).getPropertyHandle( DataSetHandle.COLUMN_HINTS_PROP ).iterator( );
			while ( iter2.hasNext( ) )
			{
				ColumnHintHandle hint = (ColumnHintHandle) iter2.next( );
				if ( columnName.getText( ).equals( hint.getAlias( ) )
						|| columnName.getText( ).equals( hint.getColumnName( ) ) )
				{
					return false;
				}
			}

			return true;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.birt.report.designer.data.ui.dataset.PropertyHandleInputDialog#rollback()
		 */
		protected void rollback( )
		{
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.birt.report.designer.data.ui.dataset.PropertyHandleInputDialog#getTitle()
		 */
		protected String getTitle( )
		{
			if ( this.getStructureOrHandle( ) instanceof Structure )
			{
				return Messages.getString( "DataSetComputedColumnsPage.toolTipText.New" );
			}
			return Messages.getString( "DataSetComputedColumnsPage.toolTipText.Edit" );//$NON-NLS-1$
		}

	}

}
